'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _utils;

function _load_utils() {
  return _utils = require('../utils');
}

var _globalConfig;

function _load_globalConfig() {
  return _globalConfig = require('../api/consumer/lib/global-config');
}

var _constants;

function _load_constants() {
  return _constants = require('../constants');
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../bit-id');
}

var _logger;

function _load_logger() {
  return _logger = _interopRequireDefault(require('../logger/logger'));
}

var _linkContent;

function _load_linkContent() {
  return _linkContent = require('./link-content');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * a dependency component may have multiple files required by the main component.
 * this class generates the link content of one file of a dependency.
 * @see RelativePath docs for more info
 */
var DependencyFileLinkGenerator = function () {
  function DependencyFileLinkGenerator(_ref) {
    var consumer = _ref.consumer,
        component = _ref.component,
        componentMap = _ref.componentMap,
        relativePath = _ref.relativePath,
        dependencyId = _ref.dependencyId,
        dependencyComponent = _ref.dependencyComponent,
        createNpmLinkFiles = _ref.createNpmLinkFiles,
        targetDir = _ref.targetDir;

    _classCallCheck(this, DependencyFileLinkGenerator);

    this.consumer = consumer;
    this.component = component;
    this.componentMap = componentMap;
    this.relativePath = relativePath;
    this.dependencyId = dependencyId;
    this.dependencyComponent = dependencyComponent;
    this.createNpmLinkFiles = createNpmLinkFiles;
    this.targetDir = targetDir;
    this.isLinkToPackage = this.createNpmLinkFiles || !this.component.dependenciesSavedAsComponents;
  }

  _createClass(DependencyFileLinkGenerator, [{
    key: 'generate',
    value: function generate() {
      this.linkFiles = [];
      if (this.component.dependenciesSavedAsComponents) {
        this.dependencyComponentMap = this.consumer.bitMap.getComponent(this.dependencyId);
      }
      this.relativePathInDependency = (_path || _load_path()).default.normalize(this.relativePath.destinationRelativePath);
      this.hasDist = this.component.dists.writeDistsFiles && !this.component.dists.isEmpty();
      if (this.relativePath.isCustomResolveUsed) {
        return this.generateForCustomResolve();
      }
      var linkFile = this.prepareLinkFile({
        linkPath: this.getLinkPath(),
        relativePathInDependency: this.relativePathInDependency,
        depRootDir: this._getDepRootDir()
      });
      this.linkFiles.push(linkFile);

      if (this.hasDist) {
        this.generateForDist();
      }

      return this.linkFiles;
    }
  }, {
    key: 'generateForCustomResolve',
    value: function generateForCustomResolve() {
      var distRoot = this._getDistRoot();
      var relativeDistPathInDependency = this._getRelativeDistPathInDependency();
      var relativeDistExtInDependency = (0, (_utils || _load_utils()).getExt)(relativeDistPathInDependency);
      var depRootDir = this._getDepRootDir();
      var depRootDirDist = this._getDepRootDirDist();

      var isCustomResolvedWithDistInside = Boolean(this.relativePath.isCustomResolveUsed && depRootDirDist && this.consumer.shouldDistsBeInsideTheComponent() && this.hasDist);
      var isCustomResolvedWithDistAndNpmLink = Boolean(this.relativePath.isCustomResolveUsed && this.consumer.shouldDistsBeInsideTheComponent() && this.hasDist && this.isLinkToPackage);

      var relativePathInDependency = isCustomResolvedWithDistInside || isCustomResolvedWithDistAndNpmLink ? `${(0, (_utils || _load_utils()).getWithoutExt)(this.relativePathInDependency)}.${relativeDistExtInDependency}` : this.relativePathInDependency;

      var linkFile = this.prepareLinkFile({
        linkPath: this.getLinkPathForCustomResolve(relativeDistExtInDependency),
        relativePathInDependency,
        depRootDir: isCustomResolvedWithDistInside ? depRootDirDist : depRootDir
      });

      if (this.relativePath.isCustomResolveUsed && this.createNpmLinkFiles) {
        linkFile.postInstallLink = true;
      }
      this.linkFiles.push(linkFile);

      if (this.hasDist && !this.consumer.shouldDistsBeInsideTheComponent()) {
        // when isCustomResolvedUsed, the link is generated inside node_module directory, so for
        // dist inside the component, only one link is needed at the parentRootDir. for dist
        // outside the component dir, another link is needed for the dist/parentRootDir.
        var importSourcePath = this._getImportSourcePathForCustomResolve(relativeDistExtInDependency);
        var linkFileInNodeModules = this.prepareLinkFile({
          linkPath: (_path || _load_path()).default.join(distRoot, importSourcePath),
          relativePathInDependency: relativeDistPathInDependency,
          depRootDir: depRootDirDist
        });
        this.linkFiles.push(linkFileInNodeModules);
      }

      return this.linkFiles;
    }
  }, {
    key: 'generateForDist',
    value: function generateForDist() {
      var distRoot = this._getDistRoot();
      var relativeDistPathInDependency = this._getRelativeDistPathInDependency();
      var relativeDistExtInDependency = (0, (_utils || _load_utils()).getExt)(relativeDistPathInDependency);
      var sourceRelativePathWithCompiledExt = `${(0, (_utils || _load_utils()).getWithoutExt)(this.relativePath.sourceRelativePath)}.${relativeDistExtInDependency}`;
      var linkFileInDist = this.prepareLinkFile({
        linkPath: (_path || _load_path()).default.join(distRoot, sourceRelativePathWithCompiledExt), // Generate a link file inside dist folder of the dependent component
        relativePathInDependency: relativeDistPathInDependency,
        depRootDir: this._getDepRootDirDist()
      });
      this.linkFiles.push(linkFileInDist);
    }
  }, {
    key: 'prepareLinkFile',
    value: function prepareLinkFile(_ref2) {
      var linkPath = _ref2.linkPath,
          relativePathInDependency = _ref2.relativePathInDependency,
          depRootDir = _ref2.depRootDir;

      var mainFile = this.dependencyComponent.dists.calculateMainDistFile(this.dependencyComponent.mainFile);
      var actualFilePath = depRootDir ? (_path || _load_path()).default.join(depRootDir, relativePathInDependency) : relativePathInDependency;
      if (relativePathInDependency === mainFile) {
        actualFilePath = depRootDir ? (_path || _load_path()).default.join(depRootDir, mainFile) : mainFile;
      }
      var relativeFilePath = (_path || _load_path()).default.relative((_path || _load_path()).default.dirname(linkPath), actualFilePath);
      var importSpecifiers = this.relativePath.importSpecifiers;
      var linkContent = this.getLinkContent(relativeFilePath);
      (_logger || _load_logger()).default.debug(`prepareLinkFile, on ${linkPath}`);
      var linkPathExt = (0, (_utils || _load_utils()).getExt)(linkPath);
      var isEs6 = Boolean(importSpecifiers && linkPathExt === 'js');
      var symlinkTo = linkContent ? undefined : actualFilePath;
      return { linkPath, linkContent, isEs6, symlinkTo };
    }
  }, {
    key: 'getLinkContent',
    value: function getLinkContent(relativeFilePath) {
      if (this.isLinkToPackage) {
        // this is used to convert the component name to a valid npm package  name
        var packagePath = `${(0, (_globalConfig || _load_globalConfig()).getSync)((_constants || _load_constants()).CFG_REGISTRY_DOMAIN_PREFIX) || (_constants || _load_constants()).DEFAULT_REGISTRY_DOMAIN_PREFIX}/${this.dependencyId.toStringWithoutVersion().replace(/\//g, '.')}`;
        return (0, (_linkContent || _load_linkContent()).getLinkToPackageContent)(relativeFilePath, packagePath);
      }
      return (0, (_linkContent || _load_linkContent()).getLinkToFileContent)(relativeFilePath, this.relativePath.importSpecifiers);
    }
  }, {
    key: 'getTargetDir',
    value: function getTargetDir() {
      if (this.targetDir) return this.targetDir;
      // when running from bit build, the writtenPath is not available
      if (!this.component.writtenPath) return this.consumer.toAbsolutePath(this.componentMap.rootDir);
      // $FlowFixMe this.component.writtenPath must be set, see the previous line
      if ((_path || _load_path()).default.isAbsolute(this.component.writtenPath)) return this.component.writtenPath;
      return this.consumer.toAbsolutePath(this.component.writtenPath);
    }
  }, {
    key: 'getLinkPath',
    value: function getLinkPath() {
      var sourceRelativePath = this.relativePath.sourceRelativePath;
      var parentDir = this.getTargetDir();
      return (_path || _load_path()).default.join(parentDir, sourceRelativePath);
    }
  }, {
    key: 'getLinkPathForCustomResolve',
    value: function getLinkPathForCustomResolve(relativeDistExtInDependency) {
      var parentDir = this.getTargetDir();
      var importSourcePath = this._getImportSourcePathForCustomResolve(relativeDistExtInDependency);
      // if createNpmLinkFiles, the path will be part of the postinstall script, so it shouldn't be absolute
      return this.createNpmLinkFiles ? importSourcePath : (_path || _load_path()).default.join(parentDir, importSourcePath);
    }
  }, {
    key: '_getDistRoot',
    value: function _getDistRoot() {
      // $FlowFixMe probably fine, should be for imported
      return this.component.dists.getDistDirForConsumer(this.consumer, this.componentMap.rootDir);
    }
  }, {
    key: '_getRelativeDistPathInDependency',
    value: function _getRelativeDistPathInDependency() {
      var relativeDistPathInDependency = (0, (_utils || _load_utils()).searchFilesIgnoreExt)(this.dependencyComponent.dists.get(), this.relativePathInDependency, 'relative');
      return relativeDistPathInDependency // $FlowFixMe relative is defined
      ? relativeDistPathInDependency.relative : this.relativePathInDependency;
    }
  }, {
    key: '_getImportSourcePathForCustomResolve',
    value: function _getImportSourcePathForCustomResolve(relativeDistExtInDependency) {
      // $FlowFixMe relativePath.importSource is set when isCustomResolveUsed
      var importSource = this.relativePath.importSource;
      var importSourceFileExt = relativeDistExtInDependency || (_path || _load_path()).default.extname(this.relativePath.sourceRelativePath);
      // e.g. for require('utils/is-string'), the link should be at node_modules/utils/is-string/index.js
      var importSourceFile = (_path || _load_path()).default.extname(importSource) ? importSource : (_path || _load_path()).default.join(importSource, `${(_constants || _load_constants()).DEFAULT_INDEX_NAME}.${importSourceFileExt}`);
      return (_path || _load_path()).default.join('node_modules', importSourceFile);
    }
  }, {
    key: '_getRelativeDepRootDir',
    value: function _getRelativeDepRootDir() {
      if (!this.dependencyComponentMap) return undefined;
      return this.dependencyComponentMap.rootDir || '.';
    }
  }, {
    key: '_getDepRootDir',
    value: function _getDepRootDir() {
      var consumerPath = this.consumer.getPath();
      var rootDir = this._getRelativeDepRootDir();
      return rootDir ? (_path || _load_path()).default.join(consumerPath, rootDir) : undefined;
    }
  }, {
    key: '_getDepRootDirDist',
    value: function _getDepRootDirDist() {
      var rootDir = this._getRelativeDepRootDir();
      return rootDir ? this.dependencyComponent.dists.getDistDirForConsumer(this.consumer, rootDir) : undefined;
    }
  }]);

  return DependencyFileLinkGenerator;
}();

exports.default = DependencyFileLinkGenerator;